/******************************************************************
 *
 *   a3d.h
 *
 *   driver for ArrangePak-3D, a tool for analyzing arrangements of planes
 *   in three dimensions
 *
 *   ArrangePak-3D 
 *   version 1.04
 *   original version:   October, 2003
 *   this revision:      May 2004
 *
 *
 *   This application supports analysis of 3-dimensional planes by constructing
 *   a virtual viewing cylinder.
 *
 *   The cylinder provides a cross-sectional view of planes that intersect
 *   it, within the bounds of the cylinder. Calculations are made on the 
 *   entire plane (beyond the bounds of the viewing cylinder).
 *
 *------------------------------------------------------------------------
 *
 *   copyright (c) 2004, 2003 Stephen Wismath, Ph.D.
 *   University of Lethbridge
 *   Lethbridge, AB   CANADA
 *
 *------------------------------------------------------------------------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *------------------------------------------------------------------------
 *
 *   Licence terms do not apply to the Library of Efficient Data Types and 
 *   Algorithms, which is licensed separately by 
 *       Algorithmic Solutions Software Gmbh 
 *       http://www.algorithmic-solutions.com/
 *
 *
 *****************************************************************/

#ifndef _A3D_PLANES__
#define _A3D_PLANES_

#include <LEDA/segment.h>
#include "a3_Plane.h"

int setupScreen(leda_window &s);          // get screen dimensions and setup window size

int redrawWindow(leda_window &, int);     // erase and redraw the entire window contents

int refresh(leda_window &, int);          // refresh main graph window
 
int redrawLines(leda_window &, int);      // redraw the lines in the main graph window

void refreshCursorCoord(leda_window&, double, double);

void refreshMessage(leda_window&, const leda_string&);

void popUpMenu(a3_Plane&, leda_window&, double, double);

// generate random arrangement of planes
void generatePlanes(a3_Plane &, leda_window &);

int getPlaneQuanFromUser(const int);


// generate LEDA_graph, GLuskap export file or Node file output
void generateOutput(leda_window&, const int);  


// add random plane to current arrangement
int addRandomPlane(a3_Plane &, leda_window &);


// add plane with default x-y orientation (z=0 for all points in the plane)
void addDefaultPlane(a3_Plane &, leda_window &);


// remove a plane from the arrangement
void deletePlane(a3_Plane &, leda_window &);

// manipulate planes
void dragPlanes(a3_Plane &, leda_window &);
leda_segment drawPlane(leda_window &, leda_d3_plane&, leda_segment&, const int&);

void showAllPlanes(leda_window &);

void manipulate_3D_Plane(leda_window &, leda_d3_plane &, leda_point &, 
			 leda_d3_segment &, const int &, const bool &);

leda_segment get_2D_Axis(const leda_d3_plane&);

void getGrabPoint(const leda_d3_plane &, const leda_segment&,
		  leda_segment &, leda_point &, leda_point &,
		  leda_d3_segment &, double &, int &, bool&);

leda_d3_point convertTo3D(const leda_point &, int);

// save current arrangement to a file
void saveToFile(leda_window &);

// load arrangement of planes from a file
void readFromFile(leda_window &);



// file handling
void setPath(leda_window &);
int getFileNameFromUser(leda_window &, leda_string &);



// open help text file in separate window panel
void HelpFile(leda_string);

void showHelpPanel(bool &, ifstream &);

// display info about this application
void displayAbout(); 


void displaySplashPanel();

#endif







