/****************************************************************
 *
 *
 *   aPlane.cpp
 *
 *   implementation for the aPlane class
 *  
 *
 *   ArrangePak-3D 
 *   version 1.04
 *   original version:   October, 2003
 *   this revision:      May 2004
 *
 *------------------------------------------------------------------------
 *
 *   copyright (c) 2004, 2003 Stephen Wismath, Ph.D.
 *   University of Lethbridge
 *   Lethbridge, AB   CANADA
 *
 *------------------------------------------------------------------------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *------------------------------------------------------------------------
 *
 *   Licence terms do not apply to the Library of Efficient Data Types and 
 *   Algorithms, which is licensed separately by 
 *       Algorithmic Solutions Software Gmbh 
 *       http://www.algorithmic-solutions.com/
 *
 *
 *****************************************************************/

#include "aPlane.h"

#include <LEDA/circle.h>
#include <LEDA/d3_point.h>
#include <LEDA/vector.h>
#include <LEDA/plane.h>
#include <LEDA/line.h>
#include <LEDA/d3_line.h>
#include <LEDA/d3_plane.h>

extern leda_point originPt;
extern double mainCircleRadius;
  


/******************************************************
 *
 *  default constructor
 *
 ****************************************************/
aPlane::aPlane()
{ 


  listIndex = 0;
}


/****************************************************
 *
 *  constructor for aPlane data type
 *
 ***************************************************/
aPlane::aPlane(const leda_d3_plane& d3Plane,
	       const leda_line& planeLine,
	       int indexNum)
{
  plane3d = d3Plane;
  viewLine = planeLine;
  listIndex = indexNum;
}


/***************************************************
 * 
 *  do nothing constructor
 *
 *  LEDA memory manager handles destruction of
 *  LEDA data types
 *
 **************************************************/
aPlane::~aPlane() { }




void aPlane::operator = (const aPlane& aPlaneInput)
{
  plane3d = aPlaneInput.plane3d;
  viewLine = aPlaneInput.viewLine;
  listIndex = aPlaneInput.listIndex;

}



void aPlane::setPlane(const leda_d3_plane & new_3d_plane)
{
  
  // start with a unit vector, normal to the default (horizontal) plane
  // adjusted from the LEDA origin to our universe origin
  leda_d3_point p1(originPt.xcoord(), originPt.ycoord(), 0);
  leda_vector v1(0.0, 1.0, 0.0);
  leda_d3_plane defPlane(p1, v1);
  leda_d3_point q, r;
  leda_list<leda_point> pointList;
  

  if (defPlane.intersection(new_3d_plane, q, r ) == 1) {
    leda_d3_line line1(q,r);
    
    leda_line line2;                // line2 will be the 2D line of intersection between plane and circle
    if (line1.project_xy(line2)) {

      // now test whether the 2D line intersects the viewport circle
      // if true, the plane is valid for use in our arrangement
      // otherwise, the plane is outside our universe, so reject it
      leda_circle circ(originPt, mainCircleRadius);
      pointList = circ.intersection(line2);
      
      if (pointList.size() == 2) {
	leda_line newLine(pointList.front(), pointList.back());
	
	plane3d = new_3d_plane;
	viewLine = newLine;
      }
    }
  }
  
  return;
}
