/******************************************************************
 *
 *   graphNode.cpp
 *
 *   user defined data type supports main application
 *  
 *   ArrangePak-3D 
 *   version 1.04
 *   original version:   October, 2003
 *   this revision:      May 2004
 *
 *
 *------------------------------------------------------------------------
 *
 *   copyright (c) 2004, 2003 Stephen Wismath, Ph.D.
 *   University of Lethbridge
 *   Lethbridge, AB   CANADA
 *
 *------------------------------------------------------------------------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *------------------------------------------------------------------------
 *
 *   Licence terms do not apply to the Library of Efficient Data Types and 
 *   Algorithms, which is licensed separately by 
 *       Algorithmic Solutions Software Gmbh 
 *       http://www.algorithmic-solutions.com/
 *
 *
 *****************************************************************/

#include "graphNode.h"


graphNode::graphNode()
{
  a = 0;
  b = 0;
  c = 0;
}



/******************************************************************
 *
 *  constructor for a graphNode data type
 *  attributes are the cardinal labels representing three planes
 *   that intersect to form the node
 *
 *  It is CRITICAL that the labels be in ascending order.
 *  But they are not always given in order, so the 
 *   values are sorted in the constructor.
 *
 *****************************************************************/
graphNode::graphNode(int aa, int bb, int cc)
{ 
  int temp;

  if (aa > bb) {
    temp = bb;
    bb = aa;
    aa = temp;
  }

  if (aa > cc) {
    temp = cc;
    cc = aa;
    aa = temp;
  }

  if (bb > cc) {
    temp = cc;
    cc = bb;
    bb = temp;
  }

  a = aa;
  b = bb;
  c = cc;
}

 

/***************************************************************
 *
 * copy constructor for graphNode
 *
 **************************************************************/ 
graphNode::graphNode(const graphNode &gIn)
{
  a = gIn.a;
  b = gIn.b;
  c = gIn.c;
}


graphNode::~graphNode() 
{

}



bool graphNode::operator ==(const graphNode &inNode)const 
{
  if (a==inNode.a && b==inNode.b && c==inNode.c)
    return true;
  return false;
}
  
bool graphNode::operator !=(const graphNode &inNode)const 
{
  if (a==inNode.a && b==inNode.b && c==inNode.c)
    return false;
  return true;
}


istream& operator >>(istream &is, graphNode &g)
{
  is >> g.a >> g.b >> g.c;
  return is;
}

ostream& operator <<(ostream &os, const graphNode &g)
{
  os << g.a << "." << g.b << "." << g.c << endl;
  return os;
}
  
