/******************************************************************
 *
 *   parameterNode.h
 *
 *
 *   user defined data type supports main application
 *  
 *    
 *   ArrangePak-3D 
 *   version 1.04
 *   original version:   October, 2003
 *   this revision:      May 2004
 *
 *
 *   defines a graph node which contains user-defined data
 *     - enumerated tag for each of 3 planes which intersect to define the vertex
 *     - 3D point in space, corresponding to the intersection of 3 planes
 *     - a plane in 3D space which contains the point
 *
 *   'parameterNode' and 'graphNode' data types are similar, except
 *   a 'parameterNode' contains two additional parameters
 *
 *------------------------------------------------------------------------
 *
 *   copyright (c) 2004, 2003 Stephen Wismath, Ph.D.
 *   University of Lethbridge
 *   Lethbridge, AB   CANADA
 *
 *------------------------------------------------------------------------
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *------------------------------------------------------------------------
 *
 *   Licence terms do not apply to the Library of Efficient Data Types and 
 *   Algorithms, which is licensed separately by 
 *       Algorithmic Solutions Software Gmbh 
 *       http://www.algorithmic-solutions.com/
 *
 *
 *****************************************************************/

#ifndef _PARA_NODE_
#define _PARA_NODE_

#include <LEDA/d3_plane.h>    // includes LEDA::d3_point and LEDA::d3_line
#include <LEDA/memory.h>

#include "graphNode.h"

class parameterNode
{
 public:
  graphNode gN;
  leda_d3_point pt;    // the point of intersection
  leda_d3_plane pl;    // plane containing the point
                       // we use this for sorting the vertices
  
  parameterNode();
  parameterNode(graphNode, leda_d3_point, leda_d3_plane);
  ~parameterNode();

  friend istream& operator >>(istream &is, parameterNode &pN);
  friend ostream& operator <<(ostream &os, const parameterNode &pN);
  
  LEDA_MEMORY(parameterNode)

};

#endif
